/* InspectAxes.h		by Paul Kunz	December 1991
 * Controls the axes of the plots
 *
 * Copyright (C)  1991  The Board of Trustees of
 * The Leland Stanford Junior University.  All Rights Reserved.
 */

#define INSPECTAXES_H_ID "$Id: InspectAxes.h,v 1.4 1992/03/09 01:45:12 pfkeb Rel $"

#import "InspectBase.h"

#import "hippo.h"

@interface InspectAxes:InspectBase
{    
    id	rbXForm;	/* Form for X-axis rebinning and display */
    id	rbYForm;	/* Form for Y-axis rebinning and display */
    id	rbXSlider0;	/* Slider for X-axis */ 
    id	rbXSlider1;	/* Slider for X-axis */ 
    id	rbXSlider2;	/* Slider for X-axis */ 
    id	rbXSlider3;	/* Slider for X-axis */
    id	rbXSlider[4];	/* Slider matrix */
    id	rbYSlider0;	/* Slider for Y-axis */ 
    id	rbYSlider1;	/* Slider for Y-axis */ 
    id	rbYSlider2;	/* Slider for Y-axis */ 
    id	rbYSlider3;	/* Slider for Y-axis */
    id	rbYSlider[4];	/* Slider matrix */
    
    int currentDim;	/* dimension of current plot */
    
    id  autoScaleXbutton;
    id  autoScaleYbutton;
    id  logScaleXbutton;
    id  logScaleYbutton;
    id  zpXbutton;		/* Zoom/Pan mode switch for x-axis */
    id  zpYbutton;		/* Zoom/Pan mode switch for y-axis */
    id  limitCheckButton;
	
    int   nx, ny;		/* number of bins before rebinning */
    float xl, xh, xw, xoff;	/* the low/high edges, width and offset
    				 * before rebinning */
    float yl, yh, yw, yoff;	/* ditto for y */
    graphtype_t         graphtype;
    BOOL numLimitCheck;		/* status of num of bins checking */
    char textFloat[15];
}

- initInspFor:aDraw;
 /*
  * Initializes inspector object and adds itself to the Inspector
  * panel
  */
 
- setOptions;
 /*
  * sets the autoscale and log scale options to selected Plot
  */
 
- limitCheckType:sender;
  /* Responds to num limit check button by setting state of
   * number of bins limit flag
   */
   
- (int) limitCheck:(int) nbins against:(int) maxbins;
   /* Returns nbins if numLimitCheck flag is off or
    * if the flag is and nbins is less than maxbins.
    * If flag is on and nbins is greater than maxbins, 
    * then runs an alert panel which may change state
    * of numLimitCheck flag.
    */

- rebin:sender;
 /* Resonds to movement of any Slider in Panel.
  */
  
- rebinForm:sender;
  /* Responds to any possible change in the forms.
   */

- setSliders;
 /*
  * sets the sliders to current selected plot
  */

- rebinAutoYRange;
 /* Sets the range of y sliders and forms from selected plot
  */
 
- tupleSetScaleType:sender;
 
- updateView;
 /*
  * Updates the Inspector's content View
  */
  
@end
