/* InspectBase.h		by Paul Kunz	December 1991
 * Base class for tuple and plot inspectors.
 * 
 * $Id: InspectBase.h,v 1.7 1992/04/23 04:14:30 pfkeb Rel $
 *
 * Copyright (C)  1991  The Board of Trustees of
 * The Leland Stanford Junior University.  All Rights Reserved.
 */

#import <objc/Object.h>
#import "hippo.h"

@interface InspectBase:Object
{
    id 	contentBox;	/* Box containing Inspector's contentView */
    id  hDraw;		/* hippo global controler object */
    id	theInspector;	/* global NewInspector */
    id	graphicView;	/* The HGraphicView holding plot */
    id  selectedPlot;   /* The last selected Plot graphic or nil if multiple */
    id  firstPlot;	/* The first selected Plot if multiple are selected */
}

- initInspFor:aDraw;
 /*
  * Initializes inspector object  Each Inspector derived from
  * this class should message its super class with this method 
  * and after it has intialized itself, should add itself
  * to the NewInspector's list of inspectors
  */
 
- windowDidUpdate:sender;
  /*
   * Called each time an event occurs.  Updates the view if current
   * selected graphic object is a plot for which one has not
   * yet done an update.   Send message to method updateView to
   * do the updating.
   */
 
- load:aView;
 /*
  * Loads the current DrawDocument's View.
  */
  
- firstPlot;
 /*
  * returns the first plot selected
  */

- updateView;
 /*
  * Updates the Inspector's content View when a single Plot
  * is selected.   An Inspector derived from
  * this class should over-rid this method.
  */

- updateEmptySelection;
 /*
  * Updates the Inspector's content View when there is no
  * Plot selected.   An Inspector derived from
  * this class should over-ride this method.
  */

- updateMultiSelection;
 /*
  * Updates the Inspector's content View when there are multiple
  * Plots selected.   An inspector derived from this class
  * should over-rid this method.
  */
- setTuple:(ntuple) aTuple;
 /* 
  * Sets the selected n-tuple in case a derived Inspector needs it.
  */
@end
