/* InspectCut.h		by Mike Gravina		November 1991
 * Controls the cuts on plots
 *
 * Copyright (C)  1991  The Board of Trustees of
 * The Leland Stanford Junior University.  All Rights Reserved.
 */ 

#define INSPECTCUT_H_ID "$Id: InspectCut.h,v 1.25 1992/04/18 20:47:40 pfkeb Rel $"

#import "InspectBase.h"

#import "Plot.h"
#import "hippo.h"

@interface InspectCut:InspectBase
{
	id	cutVal1Slider;
    	id	cutVal2Slider;
	id	sliders[2];	/* to make matrix of above */
	id 	cutValueForm;
   	id	radio;
	id	zpSwitch;	/* Zoom/Pan mode switch Button */
	id 	tpBrowser;	/* NXBrowser showing tuple data titles */
	id	tpBrowserCells;
	id 	cutBrowser;	/* NXBrowser showing cuts added */
    	id 	cutBrowserCells;
	id	showAllButton;	/* showAllState control button */
	id	addButton;	/* Button that adds cut to plot */
	
	id	docCutList;	/* List of cuts in current Document View */
        id      cutList;	/* List of cut Plots */
        id	cutPlot;	/* current cut Plot */
	id	dependList; 	/* List of Plots dependent on curent cut */
	float	varlo;		/* lower limit of cut variable */
	float	varhi;		/* upper limit of cut variable */
	
	ntuple	selectedTuple;
	cutParmType parms;
    	int 	cutButton;
	BOOL	showAllState;	/* = YES if showing all cuts in browser */

}
- initInspFor:aDraw;
 /*
  * creates a new InspectCut for global controler aDraw.
  * After initialization, adds itself to the NewInspector.
  *
  * This method is the designated initializer for the class.
  */
  
- newValueInSlider1:sender;
	/* action when slider 1 is changed */
	
- newValueInSlider2:sender;
	/* action when slider 2 is changed */
	
- newValueInForm:sender;
	/* action when a new value 1 or value2 is typed in as text */

- setSliders:(BOOL)flag;
 /*
  * Sets initial range and valudes of sliders and forms if flag is YES,
  * otherwise disables them
  */
	
- reDrawPlot: (SEL) operation inList:aList;
 /*
  * Sends message operation to each Plot object in List aList and
  * redraws them.
  */
  
- newCut:sender;
 /*
  * Creates a new cut Plot and applies it to the currently selected
  * Plots.
  */

- add:sender;
 /*
  * Adds existing selected cut to currently selected Plots
  */
  
- remove:sender;
 /*
  * Removes the cut Plot from selected Plot(s) and other plot that
  * are cutting selected Plot(s) if they also depend on the cut Plot
  */
  
- removeCut:plot;
 /*
  * Removes the cut plot from any plot that depends on it.   Method
  * used internally.
  */
  
- replace:sender;
 /*
  * Responds to replace button by replacing the selected Cut with a new
  * one.
  */
- deleteCut:plot;
 /*
  * Delete cut plot from the view's cut plot list and updates
  * panel view.
  */
  
- addCut:plot;
 /*
  * Adds the cut plot to the view's cut plot list and updates
  * panel view.
  */
  
- showButton:sender;
 /*
  * Toggles the cut Browser between showing cuts of selected plot
  * or showing all cuts
  */
   
- setTuple:(ntuple) atuple;
  	/* Sets the tuple with already prepared display held by this object
   	*/

- setShowAllState:(BOOL) state;
 /*
  * Sets the state of the cut browser to show all cuts if state is YES,
  * or show cuts on selected plot if not.
  */
  
- updateView;
 /*
  * Updates the Inspector's View when it first comes up,
  * or when there is a change
  */
  
- selectCut:sender;
 /*
  * Responds to selection in the cut Browser.
  */

- selectCutVar;
 /*
  * Sets the tuple browser and cut radio buttons to currently
  * selected cut Plot.
  */
  
- (int) browser: sender fillMatrix: matrix inColumn: (int) column;
 /* delegate method of NXBrowser.   Will modify matrix and fill it with
  * tuple data 
  */

@end
